import { SerialPort } from "serialport";
import fs from 'fs';

import * as readline from "readline";

const config = fs.readFile();

// 开启命令行用户发送mea命令会发送command.measure

export function sendCommand() {
  const port = new SerialPort({
    path: config.port,
    baudRate: config.baudRate,
  });

  // 创建readline接口
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: "串口命令> "
  });

  // 监听串口数据
  port.on("data", (data) => {
    const hexData = data.toString('hex').match(/.{1,2}/g)?.join(' ') || '';
    console.log(`收到原始数据: ${hexData}`);
    
    // 解析数据
    try {
      parseResponse(data);
    } catch (err: any) {
      console.error(`解析数据错误: ${err.message}`);
    }
  });

  // 解析响应数据
  function parseResponse(data: Buffer) {
    // 检查数据长度是否足够
    if (data.length < 9) {
      console.log("数据长度不足，无法解析温湿度信息");
      return;
    }
    
    // 从响应中提取温度和湿度
    // 假设格式为: 01 04 04 XX XX XX XX XX XX
    // 其中第4-5个字节是温度，第6-7个字节是湿度
    const responseHex = data.toString('hex').match(/.{1,2}/g) || [];
    
    // 检查响应格式
    if (responseHex.length >= 9 && responseHex[0] === '01' && responseHex[1] === '04') {
      // 提取温度值（第4-5个字节）
      const tempHex = `${responseHex[3]}${responseHex[4]}`;
      // 提取湿度值（第6-7个字节）
      const humidityHex = `${responseHex[5]}${responseHex[6]}`;
      
      // 转换成十进制
      const tempDecimal = parseInt(tempHex, 16) / 10;
      const humidityDecimal = parseInt(humidityHex, 16) / 10;
      
      console.log(`🌡️ 温度: ${tempDecimal.toFixed(1)}°C`);
      console.log(`💧 湿度: ${humidityDecimal.toFixed(1)}%`);
    } else {
      console.log("响应格式不符合预期，无法解析温湿度");
    }
  }

  // 监听串口错误
  port.on("error", (err) => {
    console.error(`串口错误: ${err.message}`);
  });

  // 串口打开成功
  port.on("open", () => {
    console.log(`成功打开串口 ${config.port}`);
    console.log("输入 'mea' 发送测量命令");
    console.log("输入 'exit' 退出程序");
    
    rl.prompt();
    
    // 处理用户输入
    rl.on("line", (line) => {
      const input = line.trim().toLowerCase();
      
      if (input === "mea") {
        console.log("发送测量命令...");
        port.write(Buffer.from(command.measure.split(' ').map(hex => parseInt(hex, 16))), (err) => {
          if (err) {
            console.error(`发送命令错误: ${err.message}`);
          } else {
            console.log("测量命令已发送");
          }
        });
      } else if (input === "exit") {
        console.log("关闭串口并退出...");
        port.close();
        rl.close();
        process.exit(0);
      } else if (input) {
        console.log(`未知命令: ${input}`);
      }
      
      rl.prompt();
    });
    
    // 处理退出
    rl.on("close", () => {
      console.log("再见!");
      port.close();
      process.exit(0);
    });
  });
}
